<?php

declare(strict_types=1);

namespace Internetgalerie\IgFrontendUser\Domain\Repository;

use Internetgalerie\IgFrontendUser\Event\AddFrontendUserConstraintsEvent;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Persistence\Repository;
use Psr\EventDispatcher\EventDispatcherInterface;

/**
 * A Frontend User repository
 */
class FrontendUserRepository extends Repository
{
    private EventDispatcherInterface $eventDispatcher;

    public function injectEventDispatcher(EventDispatcherInterface $eventDispatcher): void
    {
        $this->eventDispatcher = $eventDispatcher;
    }

    public function findBySearch(array $search, array $settings = [], bool $returnRawQueryResult = false) {
          $query = $this->createQuery();

          $constraints = [];
          $constraints  = $this->eventDispatcher->dispatch(
              new AddFrontendUserConstraintsEvent($query, $search, $settings)
          )->getConstraints();
          // Andere Sucher
          //$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('fe_users');
          //$queryBuilder->expr()->inSet('usergroup', '12');//$query->equals('usergroup', $fegroup);
          $constraints  = $this->eventDispatcher->dispatch(
              new AddFrontendUserConstraintsEvent($query, $search, $settings)
          )->getConstraints();

          if($settings['forceFrontendUserGroupUid']) {
              $fegroup = (int)$settings['forceFrontendUserGroupUid'];
              $feGroupConstrain = $query->logicalOr([
                  $query->equals('usergroup', (int)$fegroup),
                  $query->like('usergroup', (int)$fegroup . ',%'),
                  $query->like('usergroup', '%,' . (int)$fegroup),
                  $query->like('usergroup', '%,' . (int)$fegroup . ',%'),
              ]);
              $constraints[] = $feGroupConstrain;
          }

          if (isset($search['fegroup']) && $search['fegroup'] !== null && $search['fegroup'] !== '') {
              // @todo find_in_set + explode
              $fegroup = (int) $search['fegroup'];
              $feGroupHasNot = $fegroup < 0;
              $fegroup = abs($fegroup);
              // fegroup
              $feGroupConstrain = $query->logicalOr([
                  $query->equals('usergroup', (int)$fegroup),
                  $query->like('usergroup', (int)$fegroup . ',%'),
                  $query->like('usergroup', '%,' . (int)$fegroup),
                  $query->like('usergroup', '%,' . (int)$fegroup . ',%'),
              ]);
              if ($feGroupHasNot) {
                  $constraints[] =  $query->logicalNot($feGroupConstrain);
              } else {
                  $constraints[] = $feGroupConstrain;
              }
          }

          $keywordsArray = GeneralUtility::trimExplode(' ', $search['keywords'], true);
          if (!empty($keywordsArray)) {
              foreach ($keywordsArray as $keyword) {
                  $constraints[] = $query->logicalOr(
                      $query->like('username', '%' . $keyword . '%'),
                      $query->like('first_name', '%' . $keyword . '%'),
                      $query->like('last_name', '%' . $keyword . '%')
                  );
              }
          }
          if (!empty($constraints)) {
              $query->matching(
                  $query->logicalAnd($constraints)
              );
	  }
          return $query->execute($returnRawQueryResult);
      }
}
